// Definitions for HP MidRange and High-End Cellular/Partitioned systems,
// C-Class blades, and ATCA blades.  This is included by ezbmc.h and
// broken out to reduce clutter in that file, which serves as the basis
// for using any of libezbmc.

#ifndef BLADES_AND_CELLS_DOT_H
#define BLADES_AND_CELLS_DOT_H

#ifdef __cplusplus
extern "C" {
#endif

// Cellular/partitioned systems exist in one or more cabinets of varying
// types.  A cabinet enumeration (0..3) is mapped to one of these types.
// Every running system has AT LEAST 1 cabinet (even Foundation or blades).

#define EZBMC_MAX_CABS		4	// Two compute, two IOX
#define EZBMC_MAX_CELLS		16	// 8 in each compute cabinet

// PDC PAT 2.6.2, Figure 2.13, one byte.  Types are reused on Arches
#define EZBMC_CAB_TYPE_THINBOY        0x00
#define EZBMC_CAB_TYPE_FATBOY         0x01
#define EZBMC_CAB_TYPE_DOMELITE       0x02
#define EZBMC_CAB_TYPE_ORCAS          0x02	// Top range, used in <= stmts
#define EZBMC_CAB_TYPE_KEYSTONE       0x03                    // defined in PAT
#define EZBMC_CAB_TYPE_RAINIER        EZBMC_CAB_TYPE_KEYSTONE       // verified
#define EZBMC_CAB_TYPE_OLYMPIA        EZBMC_CAB_TYPE_KEYSTONE       // verified
#define EZBMC_CAB_TYPE_MATTERHORN     0x04                    // defined in PAT
#define EZBMC_CAB_TYPE_EIGER          EZBMC_CAB_TYPE_MATTERHORN     // SWAG
#define EZBMC_CAB_TYPE_IOX            0x80

// Arbitrary definitions to be consistent on non-cellular systems
#define EZBMC_CAB_TYPE_FOUNDATION     0xAA	// noncellular zx1/zx2
#define EZBMC_CAB_TYPE_C_CLASS	      0xAB	// Tahiti, Bora Bora enclosure
#define EZBMC_CAB_TYPE_CORSAIR	      0xAC	// GSTL futures

typedef struct {
	uint8_t mapping; // instance to "user" cabinet: Orca is 0, 1, 8, 9 
	uint16_t type;	 // FoundationID or EZBMC_CAB_TYPE_xxx
} CabInfo_t;

// "Top-level" discernment is defined in ezbmc.h

#define EZBMC_IS_PINNACLES(X)  (EZBMC_IS_HP(X) && (X)->Cap.ProductID == 0x1000)
#define EZBMC_IS_ARCHES(X)     (EZBMC_IS_HP(X) && (X)->Cap.ProductID == 0x1001)

#define EZBMC_IS_EIGER(X)      (EZBMC_IS_PINNACLES(X) && \
				 (X)->Cap.PlatformID == EZBMC_CAB_TYPE_EIGER)
#define EZBMC_IS_OLYMPIA(X)    (EZBMC_IS_PINNACLES(X) && \
				 (X)->Cap.PlatformID == EZBMC_CAB_TYPE_OLYMPIA)
#define EZBMC_IS_SUPERDOME(X)  (EZBMC_IS_PINNACLES(X) && \
				 (X)->Cap.PlatformID <= EZBMC_CAB_TYPE_ORCAS)

// Bugzilla 4311 needed these.  Note that CELLULAR excludes Windjammer

#define EZBMC_IS_MITTELHORN(X) (EZBMC_IS_ARCHES(X) && \
				 (X)->Cap.PlatformID == EZBMC_CAB_TYPE_EIGER)
#define EZBMC_IS_KONA(X)       (EZBMC_IS_ARCHES(X) && \
				 (X)->Cap.PlatformID == EZBMC_CAB_TYPE_OLYMPIA)
#define EZBMC_IS_SANDDUNE(X)   (EZBMC_IS_ARCHES(X) && \
				 (X)->Cap.PlatformID <= EZBMC_CAB_TYPE_ORCAS)

#define EZBMC_IS_CELLULAR_LOW(X) \
	((EZBMC_IS_PINNACLES(X) || EZBMC_IS_ARCHES(X)) && \
		 (X)->Cap.PlatformID == EZBMC_CAB_TYPE_EIGER)

#define EZBMC_IS_CELLULAR_MID(X) \
	((EZBMC_IS_PINNACLES(X) || EZBMC_IS_ARCHES(X)) && \
		 (X)->Cap.PlatformID == EZBMC_CAB_TYPE_OLYMPIA)

#define EZBMC_IS_CELLULAR_HIGH(X) \
	((EZBMC_IS_PINNACLES(X) || EZBMC_IS_ARCHES(X)) && \
		 (X)->Cap.PlatformID <= EZBMC_CAB_TYPE_ORCAS)

typedef struct {
	int		PartNum;	/* -1 == not a cellular system */
	char		ComplexName[32];
	char		PartName[64];			
	SDR_entry_t	*GroupA,	/* Property assignments */
			*reconfig_data,
			*reconfig_state,
			*GroupC;
	// Only cells in current partition have non-null values.  If
	// EZBMC_FILL_CELLINFO was clear on BMC_open, it's just a flag.
	// Otherwise, it's a pointer to the CellInfo_t for that cell.
	void		*CellInfo[EZBMC_MAX_CELLS];
} Partition_header_t;

// Blades

#define EZBMC_IS_TAHITI(X)   (EZBMC_IS_HP(X) && (X)->Cap.ProductID == 0x1201)

#define EZBMC_IS_BORABORA(X) (EZBMC_IS_HP(X) && (X)->Cap.ProductID == 0x1202)

#define EZBMC_IS_INTEGRITY_BLADE(X) \
	(EZBMC_IS_TAHITI(X) || EZBMC_IS_BORABORA(X) || EZBMC_IS_KAUAI(X))

typedef struct {			// Blade info
        char name[64];			// As applicable
	int baseSlot;			// Match exterior label
	int usedSlots;			// usually just one
        int Hslots, Wslots, 		// Physical dimension in "slots"
	    Hmm, Wmm, Dmm, 		// Physical dimension in millimeters
	    Pmin, Pmax;			// Min and max power draw in watts
	struct {			// Enclosure info, if known
        	char name[64];
        	char model[64];
        	char serial[32];
		int maxSlots;
        	struct in_addr IP;
		struct __attribute__((packed)){
        		uint8_t header;
        		struct in6_addr address;
        		uint8_t prefix;
		}IPv6[16];		// IPv6 info EVMSTR v3.27
        	uint8_t MAC[6];
		char URL[64];
        	char RackName[64];	// cClass only: the ProLiant "rack"
		char HostName[48];	// the enclosure manager
		char RackUUID[32];
                char ProductID[32];
		FPU_t FWversion;
	} enc;
} Blade_header_t;			// NO POINTERS IN HERE!!!

#ifdef __cplusplus
}
#endif
#endif
