#ifndef _EZBMC_EVENTS_H
#define _EZBMC_EVENTS_H

#ifdef __cplusplus
extern "C" {
#endif

// HP OEM type E0 events: severity levels from "Event EAS" section 4.1.6
// Matches wording of 02 events in CED
#define EZBMC_CED_ALERT_MINOR_FP            0
#define EZBMC_CED_ALERT_MAJOR_FP            1
#define EZBMC_CED_ALERT_INFORMATIONAL       2
#define EZBMC_CED_ALERT_WARNING             3
#define EZBMC_CED_ALERT_CRITICAL            5
#define EZBMC_CED_ALERT_FATAL               7

// data type for SEL and FPL.  SEL only uses lower 16 bits of record IDs,
// but FPL uses 32, which is why they're separate fields.  The raw bytes
// of a SEL/FPL entry (everything after the RecID) starts at RecType.
// You can memcpy a uint8_t raw[EZMBC_SEL_BODY_SIZE] if you want them.

#define EZBMC_SEL_BODY_SIZE	14

// Bug 8239: packed is not "inherited" in sub-structures and unions.
// Although the union only consumes 13 bytes of accessible data, 
// physloc used to start 16 bytes after the union start.  Thus physloc
// was ALWAYS unaligned (thanks to RecType).  BMC_physloc_parse()
// does the right thing, and most uses of physloc are byte-fields.
// Even though, keep the struct word-aligned at a minimum, to avoid
// future problems (eg, with BMC_physloc_NULL_test).  Bump the soname.

typedef struct __attribute__((packed)) {
    uint32_t NextID;	// Always returned when reading a SEL entry
    uint32_t ThisID;	// First two bytes of the real 16 (FPL == 4 bytes)
    time_t   Timestamp;	// Extracted from event or 0 (Unix epoch)
    void     *extra;	// Attach something of your own
    // Body of EZBMC_SEL_BODY_SIZE bytes starts here
    uint8_t  RecType;	// Which variant to choose
    union __attribute__((packed)) {
	struct __attribute__((packed)) {
	    uint32_t Timestamp;
	    uint8_t SWIDflag:1,		// Was GenID1
	    	    SWID_Slave:7,
		    LUN:2,		// Was GenID2
		    reserved:2,
		    Channel:4,
		    EvMRev,
		    SensorType,
		    SensorNum,
		    EvType:7,
		    EvDir:1,
		    Data1,
		    Data2,
		    Data3;
	    } t2;
    	struct __attribute__((packed)) {
	    uint8_t	RepEntID;
	    uint32_t	EventID:18;	// bitfield needs gcc >= 3
	    uint32_t    reserved:5;	// different from E1
	    uint32_t	NextIDhasTime:1;
	    uint32_t	DataType:5;
	    uint32_t	AlertLevel:3;
	    uint8_t	b9;		// From SPPA Event EAS
	    uint8_t	b10;
	    uint8_t	b11;
	    uint8_t	b12;
	    uint8_t	b13;
	    uint8_t	b14;
	    uint8_t	b15;
	    uint8_t	b16;
    	} e0;
    	struct __attribute__((packed)) {
	    uint8_t	RepEntID;
	    uint32_t	EventID:18;
	    uint32_t    ExRepEntID:4;	// different from E0
	    uint32_t    reserved:1;
	    uint32_t	NextIDhasTime:1;
	    uint32_t	DataType:5;
	    uint32_t	AlertLevel:3;
	    uint8_t	b9;
	    uint8_t	b10;
	    uint8_t	b11;
	    uint8_t	b12;
	    uint8_t	b13;
	    uint8_t	b14;
	    uint8_t	b15;
	    uint8_t	b16;
    	} e1;
    };			// Anonymous union needs gcc >= 3
    physloc_t physloc;	// Invalid until a call to BMC_SEL_parse_entry
} SEL_entry_t;

// A single textual entry of the "Event Database".  Apart from a single
// field, this structure is identical for type 02 and E0 events
typedef CONST struct {
	uint64_t EventID;	// Some combo of CED ID1:2:3:4:5
        char *Summary,		// The short story (one-line phrase)
	     *Description,	// The long story: also MIB "DESCRIPTION" text
	     *Cause, 		// Another take
	     *Action;		// "Call HP" :-)
        int  RepEntIDClass;	// How to interpret SEL_entry_t.RepEntID
	int  CodeBase;		// How to choose between "duplicate" Event IDs
        int  AlertLevel;	// Guideline: if this is high enough and...
				// ...SEL_entry_t.Alert is high enough...
	struct {
            int Enabled;	// ...and this is true, then send...
	    unsigned int *EntOID; // ...ENTERPRISE parameter in a trap PDU...
	    unsigned int EntOIDlength;	// How many elements in the prefix
            unsigned int ID;	// ...with this suffix.
	    char *Name,	// OID name for the trap
	         *StrDetails,	// Contents of the varbind hpIpfEvtStrDetails
	         *Summary;	// MIB "--# Summary" manager directive; if ...
	     			// ...missing, use overall event Summary
	} ST;	// SNMP Trap
	struct {
	    int EventNumber;	// If non-zero, the rest are valid
	    int EventCategory;
	    char *ProbableCause,
	         *ProviderName;
	} WI;
} Event_dictionary_entry_t;

// An entire event dictionary for events of one "record type" is part of
// the CER (Common Events Repository), a snapshot of the CED.  All of
// the dictionary headers are stored in an array (currently types 02 and E0).
typedef CONST struct {	
	uint32_t IANA;		// ATCA and beyond: plugins
	int	 RecType;	// -1 == unassigned; 0 - 0xFF == IPMI
	char	 *Path;		// to CER file for this RecType
	Event_dictionary_entry_t **Entries;	// dynamic array of pointers...
	int      nEntries;	// ...that's this big
} Event_dictionary_t;

Event_dictionary_t
    *BMC_event_dictionary_get(BMC_t *bmc, uint8_t RecType);

Event_dictionary_entry_t 
    *BMC_event_dictionary_entry_get(BMC_t *bmc,
    				    Event_dictionary_t *dictionary,
				    SEL_entry_t *pSEL);
Event_dictionary_entry_t 
    *Get_Event_Dictionary_Entry(BMC_t *bmc,
                                Event_dictionary_t *dict,
                                const char *WBEM_Providername,
                                const int WBEM_Eventnumber);
int BMC_FPL_append_entry(BMC_t *bmc, SEL_entry_t *pFPL);

int BMC_FPL_get_entry(BMC_t *bmc, SEL_entry_t *pFPL);

int BMC_FPL_entries_follow(BMC_t *bmc, SEL_entry_t *pFPL);

int BMC_FPL_refresh(BMC_t *bmc);

int BMC_SEL_append_entry(BMC_t *bmc, SEL_entry_t *pSEL);

int BMC_SEL_get_entry(BMC_t *bmc, SEL_entry_t *pSEL);

int BMC_raw_2_SEL_entry(SEL_entry_t *pSEL, uint8_t raw[], int isHP);

void BMC_SEL_parse_entry(BMC_t *bmc, 
			 SEL_entry_t *pSEL, 
			 char *Desc, 
			 unsigned int size);

int BMC_SEL_entries_follow(BMC_t *bmc, SEL_entry_t *pSEL);

int BMC_SEL_refresh(BMC_t *bmc);

int BMC_SEL_status(BMC_t *bmc);

int BMC_codebase_match(BMC_t *bmc, Event_dictionary_entry_t *entry);

uint64_t BMC_SEL2EventID(SEL_entry_t *pSEL);

// IML constants

#define IML_MAX_DATA_SIZE   320 // max size of IML free form data + text
#define IML_RAW_HEADER_SIZE  28 // FIXME: is this included in MAX_DATA_SIZE?
#define IML_IPMI_RESP_SIZE    8 // IML ipmi "get" cmd response and error return

// We size the maximum response size for BMC_CmdRsp() based on the largest
// IML event response, since currently that is the largest ipmi request
// supported by ezbmc (for now). Note, the +4 is to allow for at least the
// one byte in the condition code return.

#define IPMI_MAX_RESPONSE_SIZE (IML_MAX_DATA_SIZE + IML_RAW_HEADER_SIZE \
				+ IML_IPMI_RESP_SIZE + 4)

// data type for IML. 
// NOT CONST as the user can cobble up their own.
				
typedef struct {
   uint32_t ThisID;              /* Unique event number (can wrap) */
   uint32_t dwCount;             /* Occurrence count (at least 1) */
   time_t   InitTime;		 /* Time of initial event */
   time_t   UpdateTime;    	 /* Time of last update */
   uint8_t  bHdrRev;             /* Header version (see EVT_LOG_REVISION) */
   uint8_t  bSeverity;           /* Event severity code */
   uint16_t wClass;              /* Event class or sub-system */
   uint16_t wCode;               /* Event code for event in the class */
   uint16_t wDataLength;	 /* Length of Data field */
   uint8_t  Data[IML_MAX_DATA_SIZE]; /* event specific */
} IML_entry_t;

int BMC_IML_get_entry(BMC_t *bmc, IML_entry_t *pIML, uint16_t MatchCode);

int BMC_IML_append_entry(BMC_t *bmc, IML_entry_t *pIML, 
				uint16_t MatchCode, uint32_t *pEventNumber);

int BMC_IML_modify_entry(BMC_t *bmc, IML_entry_t *pIML);

void BMC_IML_clear_entire_log(BMC_t *bmc, uint8_t KeepEventNumber);

int BMC_EV_get(BMC_t *bmc, char *EVname, uint8_t *out, uint16_t *outsize);

int BMC_EV_set(BMC_t *bmc, char *EVname, const uint8_t *inbuf, const uint16_t insize);

int BMC_EV_get_name(BMC_t *bmc, uint16_t EVnum, uint8_t *out, uint16_t *outsize);

#ifdef __cplusplus
}
#endif

#endif
