// FRU support for access to several subtypes of information

#ifndef _EZBMC_FRU_H
#define _EZBMC_FRU_H

#include <ezbmc.h>

#ifdef __cplusplus
extern "C" {
#endif

#define	EZBMC_FRU_AREA_INTERNAL_USE	0x01
#define	EZBMC_FRU_AREA_CHASSIS_INFO	0x02
#define	EZBMC_FRU_AREA_BOARD_INFO	0x04
#define	EZBMC_FRU_AREA_PRODUCT_INFO	0x08
#define	EZBMC_FRU_AREA_MULTIRECORD	0x10
#define EZBMC_FRU_AREA_ALL		0xFF

#define PACKED __attribute__((packed))

// Stock IPMI FRU Common Header, defined in its own spec
typedef struct {
    	uint8_t	version;
    	uint8_t	InternalUse;
    	uint8_t	ChassisInfo;
    	uint8_t	BoardInfo;
    	uint8_t	ProductInfo;
    	uint8_t	MultiRecord;
    	uint8_t	zero;
    	uint8_t	checksum;
} PACKED FRU_Common_Header_t;

// JEDEC 21-C "Serial Presence Detect" (SPD) header found on DIMMs.
// The first 256 bytes are the raw data layout.
typedef struct {
    uint8_t	ManBytes;	    // byte 0 of a packed overlay
    uint8_t	SPDBytes;
    uint8_t	MemType;
    uint8_t	nRowAddrs;
    uint8_t	nColAddrs;
    uint8_t	nBanks;
    uint16_t    dataWidth;        // includes ECC bits
    uint8_t	voltageIntLevel;
    uint8_t	cycleTime;        // bits 7:4 == Ns 3:0 Ns tenths
    uint8_t	clockAccessTime;  // bits 7:4 == Ns 3:0 Ns tenths
    uint8_t	configType;       // None, Parity, ECC
    uint8_t	refreshRate;  
    uint8_t	primarySDRAMWidth;  
    uint8_t	eccSDRAMWidth;  
    uint8_t	SDRAMAttributes;  
    uint8_t	SDRAMBurstLengths;  
    uint8_t	SDRAMnBanks;  
    uint8_t	filler0[13];  
    uint8_t	Density;
    uint8_t	filler1[15];
    uint8_t     dimmHeight;          // 1 == 1.125-1.25" and 2 == 1.7"
    uint8_t	filler2[14];
    uint8_t     SPDRev;
    uint8_t     Checksum0;          /* byte 63 is sum of bytes 0-62 */
    uint8_t	JEDEC_ID[8];        /* byte 64: manufacturer ID */
    uint8_t	ManLoc;             /* location of assembly */
    uint8_t	ManPartNum[18];     /* ascii */
    uint16_t	RevCode;
    uint8_t	ManYear;            /* 2004=4, but sometimes 104 */
    uint8_t	ManWeek;
    uint32_t	AssySerialNum;
    uint8_t	ManSpecific[27];
    uint8_t	VendorSpecific1;    /* Intel definitions */
    uint8_t	VendorSpecific2;    // byte 127, end of JEDEC fixed

    // Extensions are in "HP FRU Format" if SPDBytes == 8 (rx5670 only?)

    uint8_t	FRUIDRev;		// byte 128
    uint8_t	ArtworkRevChar;
    uint8_t	ArtworkRevNum;
    uint32_t	Datecode;
    uint8_t	PartNumber[11];
    uint8_t	SerialNumber[12];
    uint8_t	FRUName[16];
    uint16_t	ScanRev;
    uint8_t	FRUSpecificInfo[16];
    uint8_t	Future[7];
    uint8_t	Checksum1;		// byte 199: covers 128-198
    uint8_t	MfgTestHistory[56];
    unsigned long int nBytes;		// byte 256: not part of JEDEC
} PACKED FRU_JEDEC_DDR2_t;

// JEDEC raw DIMM data, 200?, appendix K, DDR3.  It's called the DDR3 spec
// even though it seems to start a revision-based restatement of the
// existing 21-C spec that already covered DDR2 (and still does).
// Most of these fields are NOT direct values, they're lookup keys/indices.
// agents need nBytes, AssySerialNumber, ManPartNum, ManSpecific

typedef struct {
    struct {
    	uint8_t	BytesUsed : 4,		// byte 0
		BytesTotal: 3,
		CRCspan   : 1;
    } Ranges;
    struct {				// byte 1
    	uint8_t Additions : 4,
		Enconding : 4;
    } SPDRev;
    uint8_t	MemType,		// byte 2
		ModuleType;		// byte 3
    struct {				// byte 4
    	uint8_t	DevMbits : 4,
		Banks	 : 3,
		Res	 : 1;
    } Capacity;
    uint8_t	Addressing,		// byte 5
		VDD;			// byte 6
    struct {				// byte 7
    	uint8_t DevWidth : 3,
		Ranks	 : 3,
		Res	 : 2;
    } Org;
    struct {				// byte 8
    	uint8_t Primary	: 3,
		Ext	: 2,
		Res	: 3;
    } BusWidth;
    uint8_t	FTBDividendDivisor,
		MTBDividend,
		MTBDivisor,
		MinCycleTime,
    		pad1[17],		// Skip 13-29
		OptFeatures,
		ThermalRefreshOptions,
		ThermalSensor,
		SDRAMDevType,
		ResGen[26],
		TypeSpecific[57];	// bytes 60-116
    struct PACKED {			// bytes 117-127 (eleven total)
    	struct {
	  uint8_t IDcode : 7,
	  	  parity : 1,
		  ContCode;
    	} JEP106;
    	uint8_t	ManLoc;
    	uint16_t ManDate;
	uint32_t SerialNum;
	uint16_t CRC;
    } ModuleID;
    char	ModPartNum[18];		 // bytes 128-145
    uint16_t	ModRev,
    		ManJEDECID;
    uint8_t	ManSpecific[26];
    uint8_t	Open[80];
    unsigned long int nBytes;		// byte 256: not part of JEDEC data
} PACKED FRU_JEDEC_DDR3_t;

// Extracted from the packed binary structures outlined in the "PIROM" 
// chapter of the processor data sheets.  The fields here are synthesized
// compilations, not the actual byte layout.  It started from the
// McKinley/Montvale/Montecito PIROM structures.  I am ass-u-ming
// the QPI next-generation processors will have similar data.  If that's
// not the case, I hope to get it taken care of in here via types,
// else look for totally new structs.

typedef enum {
    CPU_TYPE_NADA = 0			// never use this
} CPU_type_t;

typedef struct {
    CPU_type_t	type;			// SWAG future
    char	FamilyDesc[24],		// "Itanium 2"
    		ModelDesc[24];		// "Madison/Mad9M"
    int		Cores;

    char	PartNumber[8],
		QDFNumber[8],
		PackageRev[8];
    uint8_t	SampleProduction;
    uint64_t	eSignature;		// from chip; also in SMBIOS
    uint8_t	ArchRev, 		// ia64: CPUID[3]
		Family, 
		Model, 
		Stepping;
    uint16_t	CoreFrequency,		// MHz
		SystemBusFrequency,	// Not on QPI/FDI processors
 		L2CacheSize,		// kbytes, ia32 only
 		L3CacheSize;		// kbytes, total on ia64
    uint8_t	PIROMrev;		// Add to end of struct
    uint16_t	BaseCoreFrequency;	// Add to end of struct
} FRU_PIROM_t;

// There is a separate IPMI document that defines a FRU layout.  It's
// a run-length encoded mashup of optional fields.  A field's presence is
// indicated via flags in the "Type-Length" field for an entry.  An ezmbc
// FRU entry is a simplified aggregation of important fields from
// multiple areas.

typedef struct {
	uint8_t	TL;	// Type-Length
	char *value;	// DYNAMICALLY ALLOCATED!!!  see BMC_FRU_free below
} FRU_item_t;

typedef struct {		// IPMI FRU spec.  Don't keep checksums
	uint8_t	RecType;
	uint8_t EOLVer;
	uint8_t RecLen;		// sizeof(raw)
	uint8_t raw[1];		// used as variable length
} PACKED FRU_MultiRecord_t;

typedef struct {	// IPMI definition of areas in a FRU
    time_t	BoardManDate;
    uint8_t	ChassisType;
    FRU_item_t 	FRUFileID, // Common to board & product areas
		ChassisPartNumber,
		ChassisSerialNumber,
		BoardManName,
		BoardProductName,
		BoardSerialNumber,
		BoardPartNumber,
		ProductManName,
		ProductName,
		ProductPartModelNumber,
		ProductVersion,
		ProductSerialNumber,
		ProductAssetTag,
		ProductCustom;
		int nMultiRecords;
		FRU_MultiRecord_t **MultiRecords;
} FRU_IPMI_t;

typedef enum {
    FRU_TYPE_NADA = 0,		// don't use this
    FRU_TYPE_IPMI_SPEC,
    FRU_TYPE_JEDEC_DDR2,
    FRU_TYPE_JEDEC_DDR3,
    FRU_TYPE_PIROM		// Unless there are more...
} FRU_type_t;

// FRU entry.  The first 7 fields are copied from the corresponding SDR.
typedef struct {
    char        Name[20];
    uint8_t	LUN, FRUID, EntityID, EntityInst, DevType, DevTypeMod;
    FRU_type_t	type;			// Which union overlay to use
    union {
    	uint8_t		 raw[128];	// for tpot, but only for DDR
	FRU_IPMI_t	 f;		// preserve legacy source compatibility
	FRU_JEDEC_DDR2_t DDR2;		// BREAK legacy source compatibility
	FRU_JEDEC_DDR3_t DDR3;		// new
	FRU_PIROM_t	 p;		// preserve legacy source compatibility
    };
} FRU_entry_t;

int BMC_FRU_read(BMC_t *bmc, 		// Avoid memory leaks with BMC_FRU_free
		 SDR_entry_t *pSDR, 
		 uint8_t areas, 
		 FRU_entry_t *pFRU);

int BMC_FRU_read_Common_Header(BMC_t *bmc, 
			       SDR_entry_t *pSDR, 
			       FRU_Common_Header_t *comhdr);

int BMC_FRU_read_Area(BMC_t *bmc, 
		    SDR_entry_t *pSDR, 
		    FRU_Common_Header_t *comhdr,
		    uint8_t selected,
		    uint8_t **area,
		    uint16_t *arealen);

SDR_entry_t *BMC_FRU_match(BMC_t *bmc, 
			   uint8_t EntityID, 
			   uint8_t EntityInst, 
			   uint8_t DevType,
			   uint8_t DevTypeMod);

void BMC_FRU_free(FRU_entry_t *pFRU);	// free allocations of BMC_FRU_read

void BMC_FRU_load_pSDR_zero(SDR_entry_t *pSDR);	// Cobble up logical FRU 0

int BMC_FRU_write_ProductAssetTag(BMC_t *bmc, char newTag[]);

void BMC_FRU_copy(FRU_entry_t *dest, FRU_entry_t *src);

#ifdef __cplusplus
}
#endif

#endif
