#ifndef _EZ_PCI_H
#define _EZ_PCI_H

enum {
	MEMORY_TYPE_UNKNOWN = 1,
	MEMORY_TYPE_IO,
	MEMORY_TYPE_MAPPED,
	MEMORY_TYPE_EXPROM
};

enum {
	PCI_DEVICE_UNKNOWN = 1,
	PCI_DEVICE_ENABLED,
	PCI_DEVICE_DISABLED
};

enum {
	SLOT_TYPE_UNKNOWN = 1,
	SLOT_TYPE_PCI,
	SLOT_TYPE_PCI_X,
	SLOT_TYPE_PCI_E
};

#define PCI_TYPE0_ADDRESSES    6
#define PCI_TYPE1_ADDRESSES    2

/* PCI_getSlotNumber() return values */
#define SLOT_NOT_FOUND 0xFF


/************* structures *************/

/* decoded pci memory entries */
typedef struct PciMemoryEntry
{
    unsigned long memoryBaseAddress; /* base address for this memory entry */
    unsigned long memoryType;        /* memory type */
    unsigned long memorySize;        /* address space used */
} PCIMEMORYENTRY, * pPCIMEMORYENTRY;

/* decoded pci function entries */
typedef struct PciFunctionEntry
{
    unsigned char  classCode[3];     /* class code for this pci function:
                                           0  .. 7  -> reg level prog interface
                                           8  .. 15 -> sub-class code
                                           16 .. 23 -> base class code */
    unsigned char   classDescription[80];   /* text descr. of class */
    unsigned short  deviceId;
    unsigned short  vendorId;
    unsigned short  subDeviceId;
    unsigned short  subVendorId;
    unsigned char   revId;
    unsigned char   headerType;
    unsigned char   intLine;
    unsigned char   numMemoryEntries;
    PCIMEMORYENTRY  memoryEntries[PCI_TYPE0_ADDRESSES];
} PCIFUNCTIONENTRY, * pPCIFUNCTIONENTRY;

/* decoded pci slot */
typedef struct PciDecodedData
{
    unsigned char   boardId[8];    /* 0-not supported; 0xFFFFFFF-empty slot */
    unsigned char   boardName[160];
    unsigned char   subsystemName[160];
    unsigned char   deviceStatus;
    PCIFUNCTIONENTRY functionEntry;
} PCIDECODEDDATA, * pPCIDECODEDDATA;

/* pci slot size */
typedef struct PCIDimensions
{
    unsigned short  width;  // in mm, 0 for unknown
    unsigned short  height; // in mm, 0 for unknown
    unsigned short  length; // in mm, 0 for unknown
} PCIDIMENSIONS, * pPCIDIMENSIONS;

typedef struct PciSlotData
{
   unsigned int    cabinetNumber;     /* cabinet number for this pci dev */
   unsigned int    bayNumber;         /* bay number for this pci dev */
   unsigned int    chassisNumber;     /* chassis number for this pci dev */
   unsigned int    physicalSlot;      /* slot number the device is plugged into */
   unsigned int    domainNumber;      /* domain number for this config data */
   unsigned int    busNumber;         /* bus number for this config data */
   unsigned int    deviceNumber;      /* device number for this config data */
   unsigned int    functionNumber;    /* function number of this PCI device */
   long            softInterrupt;     /* software interrupt for this slot device,
                                         obtained thru proc/bus/pci/devices */
   unsigned short  cardClockSpeed;    /* card clockspeed available, in MHz */
   unsigned short  linkClockSpeed;    /* slot clockspeed available, in MHz */
   unsigned char   cardBitWidth;      /* 32 or 64, for the card */
   unsigned char   linkBitWidth;      /* 32 or 64, for the slot */
   unsigned char   linkLaneWidth;     /* slot physical supported mode x1,x2,x4,x16
                                         0 if non-PCIe */
   unsigned char   linkLaneMode;      /* actual running PCIe mode x1,x2,x4,x8,x16
                                         0 if non-PCIe */
   unsigned char   slotType;          /* PCI, PCI-X or PCI Express */
   char            hwLocation[256];   
   unsigned char * pciRawConfigData;  /* pointer to pci config registers,
                                         null for empty slot */
   PCIDECODEDDATA  pciDecodedData;
   PCIDIMENSIONS   physicalDimensions;/* 0 for embedded device */
   unsigned char   isEmbedded;        /* set to 1 if embedded, or set to 0 */
   unsigned char   isBridge;          /* set to 1 if PCI bridge, or set to 0 */
   unsigned char   reserved;
   unsigned char   rootComplex;       /* PCI Express RootComplex 0xFF if unavailable*/
   unsigned char   rootPort;          /* PCI Express RootPort    0xFF if unavailable*/
}PCISLOTDATA, * pPCISLOTDATA;


/* Functions */
#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************************
*    To reiterate the notes within the source of this library:      *
*       This library is NOT thread safe. Make sure to use the API within  *
*       a single-threaded section of your code.                           *
**************************************************************************/

char *GetLibVersion(void);
	/* returns the version of this ezpci library */

int IsPCILibInit(void);

int PCI_isCellular(void);

int PCI_initLib(char *pciConfigFile);
     /*   PCI_initLib must be called to initialize the library before any
          other function is used. */

int PCI_closeLib(void);
     /* Frees up all internal structures used for accounting and SLOT information. */

char *PCI_strerror(int errNum);

#define EZPCI_MATCH_ALL 0xFF       // Can be used as an argument to PCI_get* 
				   // functions for cabinet bay or chassis
				   // in order to force the PCI_get function 
                                   // to overlook filters on that argument.

int PCI_getNumDevicesAtSlot(
		unsigned int cabinet, unsigned int bay,
		unsigned int chassis, unsigned int slot, 
		int *numDev);

int PCI_getSlotDataAtSlotbyIndex(
		unsigned int cabinet, unsigned int bay,
		unsigned int chassis, unsigned int slot,
		int index,
		pPCISLOTDATA *slotData
	);


int PCI_getSlotDataByIndex(int idx, pPCISLOTDATA *slotData);

int PCI_getSlotDataByDevFn(unsigned int domain,
						   unsigned int bus,
						   unsigned int dev,
						   unsigned int fun,
						   pPCISLOTDATA *slotData);

int PCI_getSlotDataByStr(const char *pciStr , pPCISLOTDATA *slotData);

int PCI_getSlotDataByIRQ(int irq, pPCISLOTDATA *slotData);

int PCI_getNumInterfaces(int *numIntf);

int PCI_getNumPhysicalSlots(int *outSlots);
   /* PCI_getNumPhysicalSlots : returns the number of physical 
      PCI slots as seen by the firmware. On rx8620 and rx7620 I/O chassis,
      slot "0" is actually a special physical slot designed for only
      the ManagementProcessor card. So, slot 0 on these machines actually
      designates a physical slot that is not for std PCI cards. This
      function returns a number inclusive of this MP card slot.*/
    

int PCI_getNumIOChassis(int *numIOC);

int PCI_getIOChassisLocationByIndex(int idx, int *cabinet, int *bay,
                                       int *chassis);

int PCI_getSlotNumber(unsigned int domain_num,
					  unsigned char bus_num,
					  unsigned char device_num,
					  unsigned char function_num,
					  unsigned long *outSlot);
   /* notes: if you don't know the domain_num, set this arg to EZPCI_MATCH_ALL, 
	any kernels post 2.6 should have this information avail */


enum {
	//Slot Characteristics 1 and 2 defined in SMBIOS specification 3.3.10.6-7
	PROPERTY_TYPE_SLOT_CHARACTERISTICS = 1
};

// for PROPERTY_TYPE_SLOT_CHARACTERISTICS
enum {
	PCISLOT_POWER_V5_0  = 0x01,
	PCISLOT_POWER_V3_3  = 0x02,
	PCISLOT_HOTPLUGABLE = 0x04,
	PCISLOT_PME_SIGNAL  = 0x08,
	PCISLOT_SMBUS_SIGNAL= 0x10
};

int PCI_getSlotExtraPropertyAtSlot (
	   unsigned int cabinet, unsigned int bay,
	   unsigned int chassis, unsigned int slot,
	   unsigned int propertyType,
	   unsigned int * outValue
	);

#ifdef __cplusplus
}
#endif

#endif
